<?php
/* fileit.php
   Defines the Siteation filer, which allows files to be uploaded.
   Part of Siteation.  Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.config.php");
require_once("libs/site.error.php");
require_once("libs/site.lib.php");
require_once("libs/session.lib.php");
require_once("libs/user.lib.php");

// Buffer output
ob_start();

/* This code prevents client browsers and intermediate proxies from caching
   the output of this script.
*/
// Set the expiration date to a date in the past
header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
// Indicate that the content is always modified
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
// HTTP/1.1 headers that will prevent buffering
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
// HTTP/1.0 headers this will prevent buffering
header("Pragma: no-cache");

// Connect to the site database
$thisSite = new SITE();

// Start (or continue) a session
$thisSession = new SITE_SESSION($thisSite);

// This array will hold information sent by the client
$request = array();

// Holds a message for the user
$message = "";

// Retrieve the parameters passed
$request = array_merge($_GET, $_POST);
// Change all of the key names to lowercase
$request = array_change_key_case($request);

// Change the action to uppercase to avoid case conflicts
if ((isset($request["act"])) && (!empty($request["act"])))
  $request["act"] = strtoupper($request["act"]);

/* Retrieve information about the current user */
$thisUser = new SITE_USER($thisSite);
// Get the user's information
$result = $thisUser->Get($_SESSION["UserName"]);
// If the user was not found, indicate an error
if ($result < 0) 
  DiePrettily(seResource, array(
  "FILEIT",
  " - Unable to retrieve information about the user: &lt;",
  $_SESSION["UserName"], "&gt;"));
  
// This array is used to test that the action specified by the client is valid
$validActions = array (
  "UPLOAD" => "w",
  "DELETE" => "d"); 
  
// This array holds PHP specific upload errors
$uploadErrors = array(
  "The file uploaded successfully",
  "The file was too large to upload",
  "The file was too large to upload",
  "Only part of the file uploaded successfully",
  "No file was received",
  "The system was unable to allocate a temporary storage location for the uploaded file",
  "The system was unable to save the uploaded file",
  "The upload was cancelled by another process"
); 

// Retrieve the user's subscription
$subscription = $thisUser->Subscription("@FILES!");
  
/* If we don't have an array containing information about the group
   subscription to this channel, there's a problem (the user probably cannot
   access the channel)
*/
if (!is_array($subscription)) DiePrettily($subscription, array( 
  "FILEIT",
  " - The filing system was unable to determine whether or not the user -- ",
  $_SESSION["UserName"]," -- ",
  " has permission to modify files on the system."));

// If the access rights have not been specified, indicate an error
if (empty($subscription["Rights"]))
  DiePrettily(seResourceNotFound, array( 
  "FILEIT",
  " - The posting system was unable to determine whether or not the user -- ",
  $_SESSION["UserName"]," -- ",
  " has permission to modify files on the system."));

/* Match the access rights of the user against the action they wish to perform.
   If they do not have permission to perform the requested action, return an
   error
*/
if (strpos($subscription["Rights"], $validActions[$request["act"]]) === FALSE)
  DiePrettily(seResourceForbidden,
    "FILEIT",
    " - The user -- ", $_SESSION["UserName"], " -- ",
    "needs appropriate permissions in order to modify files on the system.");

/* Set the last subscription used by the user in the session variables so that
   others can use it
*/
/* Set the last subscription used by the user in the session variables so that
   others can use it
*/
$_SESSION["Subscription"] = implode_assoc($subscription);
$_SESSION["UserRights"] = $subscription["Rights"];

// Check to see if we have uploaded files
if (count($_FILES) > 0) {
  // Create a query to update the FILES table
  $thisFile = new Query($thisSite->DataSource());
  $thisFile->location = array($SITE_CONFIG["TABLE_PREFIX"]."attachments");
  // Retrieve the file that was uploaded
  $file = $_FILES["file"];
  // Ensure the file was actually uploaded (not spoofed)
  if ((!is_uploaded_file($file["tmp_name"])) || ($file["error"] != 0))
    DiePrettily(seResource, array(
    "FILEIT",
    " - The system was unable to verify that the file uploaded successfully.  ",
    $uploadErrors[$file["error"]], "."));
  // Get the base upload directory
  $filePath = $thisSite->Setting("Uploads","Destination");
  // Uploaded files are stored in a directory based on user name and date 
  $filePath .= $thisUser->Setting("UserName");
  $filePath .= "/".date("Y-m-d");
  // Replace the path separators in $filePath (in case we are running on a Windows machine)
  if (!is_dir($filePath)) {
    $result = mkdir(convert_path($filePath), 0775, true);
    if (!$result) DiePrettily(seResourceForbidden, array(
      "FILEIT",
      " - The filing system failed to create a folder to receive the file that ",
      "was uploaded.  This is most likely due to a configuration error on the ",
      "web server."));
  }
  // Move the file
  $originalFileName = expand_filename($file["name"]);
  /* To ensure that all uploaded file names are unique, the new file name 
     will consist of the current Unix time value plus its original extension */
  $fileName = $filePath;
  $fileName .= "/".bin2hex(md5(microt(), true));
  $fileName .= ".".$originalFileName["extension"];
  // Move the file to permanent storage
  $result = move_uploaded_file($file["tmp_name"], unconvert_path($fileName));
  // If we were unsuccessful, report the error
  if ($result == FALSE) DiePrettily(seQuery, array(
    "FILEIT",
    " - The file -- ", $file["name"], " -- failed to upload properly."));
  // Change the file path so it can be retrieved by the web server
  $fileURL = $thisSite->Setting("Uploads", "BaseURL");
  $fileURL .= "/".$thisUser->Setting("UserName");
  $fileURL .= date("Y-m-d");
  $fileName = $fileURL."/".basename($fileName);
  if ((!isset($request["name"])) || empty($request["name"]))
    $request["name"] = $originalFileName["name"].".".$originalFileName["extension"];
  // Store the attachment information
  $thisFile->fieldsValues = array(
    "AttachmentName" => $request["name"],
    "FileName" => $originalFileName["name"].".".$originalFileName["extension"],
    "FileType" => $file["type"],
    "FileSize" => $file["size"],
    "Description" => Scrub($request["description"]),
    "Keywords" => Scrub($request["keywords"]),
    "SourcePath" => $fileName,
    "PostedBy" => $thisUser->Setting("UserName"),
    "Extra" => ""
  );
  // Insert the attachment information
  $result = $thisFile->Insert();
  // If we were unsuccessful, indicate an error
  if ($result <= 0) DiePrettily(seQuery, array(
    "FILEIT",
    " - The filing system was unable to store tracking information for the ",
    "uploaded file.  The operation returned ", $result, "."));
}

// Redirect the client to the specified view
$redirect = "2;URL=http://".$_SERVER["HTTP_HOST"];
$redirect .= $SITE_CONFIG["SCRIPT_URL"]."/templates/site/attach.tpl.html";

// Flush all session information until the session is resumed (or it expires)
$thisSession->__destruct();
// Flush all site information
$thisSite->__destruct();

/* Output a pseudo-HTML page so that the cookie can be cleared and reset */
/* See if we can get the special template for site messages */

echo <<< UGLY_MESSAGE
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
  <meta http-equiv="content-type" content="text/html; charset=windows-1250">
  <meta http-equiv="refresh" content="$redirect">
  <link rel="stylesheet" type="text/css" href="templates/default/default.css">
  <title>Post</title>
  </head>
  <body class="PopUp">
    <div class="Post">
      <div>
        <span class="PostTitle">File Upload</span>
      </div>
      <div class="PostBody">
        $message<br />
        Your file uploaded successfully.
      </div>
      <div class="PostFooter">&nbsp;</div>
    </div> 
</html>
UGLY_MESSAGE;
// If we're debugging, show the debugging console
if(SITE_DEBUG) ShowDebugConsole();
header("HTTP/1.1 200 OK");
// Flush the output buffer
ob_end_flush();  

?>
