<?php

/* Determines whether an array is associative or not (whether it has numeric
   indexes or not)
*/
function is_assoc($array) {
  return (is_array($array) && !isset($array[0]));
}

/* Takes an array of associative values and returns a string of the form
   [array_key][$keyGlue][array_value] and delimited by $glue
*/

function implode_assoc($array, $glue = ",", $keyGlue = "=", $skipEmpty = TRUE) {
  $result = "";
  // Ensure we are not being passed an empty array
  if (is_array($array) && !empty($array)) {
    // Run through each value in the array
    foreach($array as $key => $value) {
      // Skip empty items
      if(!$skipEmpty || $value != "") {
        $result .= $key.$keyGlue.$value.$glue;
      }
    }
    // Trim any trailing glue characters from the result string
    $result = rtrim($result, $glue);
  }
  // Return the result
  return($result);
}

/* Takes a string of the form [key]=[value] that is delimited by $glue and
   returns an associative array
*/
function explode_assoc($input, $glue = ",", $keyGlue = "=") {
  $result = array();
  // Ensure we are not being passed an empty string
  if (empty($input)) return($result);
  // First, explode the string into key/value pairs
  if (is_array($input)) DiePrettily(seInvalidParam, array(
    "Explode!",
    " - The parameter passed was invalid. ",
    implode_assoc($input)));
  $explodeInput = explode($glue, $input);
  // Run through each key/value pair and separate the key from the value
  foreach($explodeInput as $keyValue) {
    $explodeKeyValue = explode($keyGlue, $keyValue);
    // Handle empty values
    if (!empty($explodeKeyValue[1]))
      $result[$explodeKeyValue[0]] = $explodeKeyValue[1];
    else $result[$explodeKeyValue[0]] = "";
  }
  // Return the result
  return($result);
}

/* Collapses an associative array in which each key corresponds to an array
   of values
*/
function collapse_assoc($input, $glue = ",", $keyGlue = ".") {
  // Continue only if we have a non-empty array
  if (!is_array($input) || empty($input)) return("");
  $result = "";
  // Iterate through the array
  foreach($input as $key => $value) {
    // Skip empty keys
    if (is_array($value) && !empty($value)) {
      foreach($value as $keyValue) $result .= $key.$keyGlue.$keyValue.$glue;
    }
  }
  // Trim any trailing glue characters
  return(rtrim($result, $glue));
}

/* Explodes a string into an associative array where each key corresponds to
   an array of values
*/
function expand_assoc($input, $glue = ",", $keyGlue = ".") {
  $result = array();
  // Ensure we are not being passed an empty string
  if (empty($input)) return($result);
  // First, explode the string into key/value pairs
  $explodeInput = explode($glue, $input);
  // Run through each key/value pair and separate the key from the value
  foreach($explodeInput as $keyValue) {
    $explodeKeyValue = explode($keyGlue, $keyValue);
    // Handle empty keys
    if (!empty($explodeKeyValue[1]))
      $result[$explodeKeyValue[0]][] = $explodeKeyValue[1];
    else $result[$explodeKeyValue[0]][] = "";
  }
  // Return the result
  return($result);
}

// Explodes a filename into an array containing the file path, file name, and extension
function expand_filename($input) {
  // Return the expanded path information
  $filename = explode(".", basename($input));
  $result = array(
    "path" => dirname($input),
    "name" => $filename[0],
    "extension" => $filename[1]);
  return($result);
}

// Collapses a file name from an array containing the file path, file name, and extension into a string
function collapse_filename($input) {
  if (is_array($input)) {
    $result = strtr($input["path"], "/", DIRECTORY_SEPARATOR);
    $result .= $input["name"].".".$input["extension"];
    return($result);
  }
}

// Converts the path separators in the specified file path to the path separators
// used by the system on which the server is running 
function convert_path($input) {
  return(strtr($input, "/", DIRECTORY_SEPARATOR));
}

function unconvert_path($input) {
  return(strtr($input, DIRECTORY_SEPARATOR, "/"));
}

/* From jsheets at shadonet dot com via php.net.
   Used to make a random string of characters.
*/
function RandomString($min_length = 32, $max_length = 64)
{
  $key = '';
  // build range and shuffle range using ASCII table
  for ($i=0; $i<=255; $i++) {
     $range[] = chr($i);
  }
  // shuffle our range 3 times
  for ($i=0; $i<=3; $i++) {
     shuffle($range);
  }
  // loop for random number generation
  for ($i = 0; $i < mt_rand($min_length, $max_length); $i++) {
     $key .= $range[mt_rand(0, count($range) - 1)];
  }
  $return = base64_encode($key);
  if (!empty($return)) {
     return $return;
  } else {
     return 0;
  }
}

// Returns a string enclosed by $enclosure
function Quote($what, $enclosure = "''") {
  // Return the enclosed string
  return($enclosure{0}.$what.$enclosure{1});
}

// Returns a string of the format <key> <relOp> '<value>'
function Expression($keyName = "", $relOp = "", $value = "") {
  return $keyName.$relOp.Quote($value);
}

// Encodes HTML entities as their XML numeric counterparts
function EncodeEntities($input) {
  $input = strtr($input, array (
    "" => "&#8220;",
    "" => "&#8221;",
    "" => "&#8216;",
    "" => "&#8217;",
    "&nbsp;" => "&#160;",
    "&iexcl;" => "&#161;",
    "&cent;" => "&#162;",
    "&pound;" => "&#163;",
    "&curren;" => "&#164;",
    "&yen;" => "&#165;",
    "&brvbar;" => "&#166;",
    "&sect;" => "&#167;",
    "&uml;" => "&#168;",
    "&copy;" => "&#169;",
    "&ordf;" => "&#170;",
    "&laquo;" => "&#171;",
    "&not;" => "&#172;",
    "&shy;" => "&#173;",
    "&reg;" => "&#174;",
    "&macr;" => "&#175;",
    "&deg;" => "&#176;",
    "&plusmn;" => "&#177;",
    "&sup2;" => "&#178;",
    "&sup3;" => "&#179;",
    "&acute;" => "&#180;",
    "&micro;" => "&#181;",
    "&para;" => "&#182;",
    "&middot;" => "&#183;",
    "&cedil;" => "&#184;",
    "&sup1;" => "&#185;",
    "&ordm;" => "&#186;",
    "&raquo;" => "&#187;",
    "&frac14;" => "&#188;",
    "&frac12;" => "&#189;",
    "&frac34;" => "&#190;",
    "&iquest;" => "&#191;",
    "&euro;" => "&#8364;"));
  return($input);
}

/* Sanitizes the specified string by passing it through strip_tags(),
   htmlentities(), and nl2br.  You can optionally specify a list of allowed tags
   which will be passed to strip_tags().  You can also specify whether the
   $input tag has line breaks.
*/
function Scrub($input, $allowedTags = "", $multipleLines = FALSE) {
  // Continue only if we have a non-empty string
  if (empty($input)) return("");
  $input = strip_tags($input, $allowedTags);
  // Replace garbage HTML inserted into the specified string
  $scrubSearch = array(
    '/(\r)+/',                      // Carriage returns
    '/(\n)+/',                      // Newlines
    '/\s?style(.*?)""/is',          // Blank style information
    '/\s?class(.*?)Mso(.*?)"/is',   // Microsoft Office classes (created when pasting from Word)
    '/<p>&nbsp;<\/p>/is',           // Empty paragraphs
  );
  $scrubReplace = array(
    ' ',
    ' ',
    '',
    '',
  );
  $input = preg_replace($scrubSearch, $scrubReplace, $input);
  // Clean up the specified string
  $input = EncodeEntities($input);
  // Return the newly-sanitized string
  return($input);
}

/* Summarizes the given string by wrapping it at the specified number of
   characters and tacking on an indicator that the given string is longer than
   the specified number of characters
*/
function Summarize($input, $maxLength = 255, $indicator="") {
  // Continue only if we have a non-empty string
  if (empty($input)) return("");
  // Strip all tags from the input
  $input = strip_tags($input);
  // Replace HTML entities with their numeric counterparts
  $input = EncodeEntities($input);
  if (strlen($input) <= $maxLength) return($input);
  $result = "";
  // Get each word in the input string
  $words = explode(" ", $input);
  // Add words until the string is in danger of being too long
  foreach($words as $word) {
    // As long as adding the word won't make the string too long
    if ((strlen($result) + strlen($word)) < $maxLength) {
      // Add the word
      $result .= $word." ";
    }
    else break;
  }
  // Return the result
  $result = trim($result);
  if (!empty($indicator)) $result .= $indicator;
  return($result);
}

/* Tokenizes a string based on the delimiters specified in $delimiters.
   Returns an array of tokens.
*/
function Tokenize($input, $delimiters) {
  // If input is empty, return nothing
  if (empty($input)) return (FALSE);
  // If $delimiters is empty, return $input unchanged
  if (empty($delimiters)) return (array($input));
  $delim = array();
  // Construct an associative array out of $delimiters
  for ($i = 0; $i < strlen($delimiters); $i++) $delim[$delimiters{$i}] = TRUE;
  // This holds an accumulated string of non-delimiter characters
  $word = $input{0};
  // This will hold our result
  $result = array("");
  // Parse every character in $input
  for ($i = 0; $i < strlen($input); $i++) {
    // Is the character a delimiter?  Is the word non-empty?
    if (isset($delim[$input{$i}])) {
      // Store the word
      if (!empty($word)) $result[] = $word;
      // Start the word over
      $word = "";
    }
    else $word .= $input{$i};
  }
  // If one word yet remains, add it
  if (!empty($word)) $result[] = $word;
  // Return the result
  return($result);
}

/* Creates a list of keywords for a specified string based on the frequency of
   words used in the string.  Returns an array of keywords.
*/
function GetKeywords($input, $minLength = 4) {
  // If input is empty, return nothing
  if ((!is_string($input)) || (empty($input))) return (FALSE);
  $keywords = array();
  // Strip all tags from the input
  $text = strip_tags(strtolower($input));
  // Replace HTML entities with their numeric counterparts
  $text = EncodeEntities($text);
  // Retrieve the frequency of words used in the text  
  $output = array_count_values(str_word_count($text, 1));
  // Sort the array
  arsort($output);
  // Leave out any keywords that are less than than the minimum length
  foreach ($output as $key => $value) {
    if (strlen($key) >= $minLength) $keywords[] = $key;
  }
  // Return the keywords
  return($keywords);
}

// Simple function to replicate PHP 5 microtime() behaviour
function microt($asFloat = false)
{
  // Return the current Unix timestamp with microseconds (as a float)
  if ($asFloat) {
     list($usec, $sec) = explode(" ", microtime());
     return ((float)$usec + (float)$sec);
  }
  else return(microtime());
}

/* This array contains recognized browser names that have been embedded in
   user agent strings.
   Provided by: http://www.zytrax.com/tech/web/browser_ids.htm
*/
$_AGENT_NAMES = array(
  "1X" => true,
  "acorn-http" => true,
  "amaya" => true,
  "amigavoyager" => true,
  "amiga-aweb" => true,
  "bison" => true,
  "bluefish" => true,
  "browsex" => true,
  "camino" => true,
  "chimera" => true,
  "check&get" => true,
  "contiki" => true,
  "curl" => true,
  "dillo" => true,
  "doczilla" => true,
  "edbrowse" => true,
  "elinks" => true,
  "emacs-w3" => true,
  "epiphany" => true,
  "fetch" => true,
  "firefox" => "Mozilla Firefox",
  "firebird" => "Mozilla Firefox",
  "galeon" => true,
  "googlebot" => true,
  "ibrowse" => true,
  "phoenix" => "Mozilla Firefox",
  "icab" => true,
  "k-meleon" => true,
  "konqueror" => "Konqueror",
  "links" => "Links",
  "lynx" => "Lynx",
  "mosaic" => "NCSA Mosaic",
  "mmosaic" => true,
  "msie" => "Microsoft Internet Explorer",
  "ncsa_mosaic" => "NCSA Mosaic",
  "netscape" => "Netscape Navigator",
  "netpositive" => true,
  "netsurf" => true,
  "omniweb" => true,
  "opera" => "Opera",
  "oregano" => "Oregano",
  "printsmart" => "HP PrintSmart",
  "retawq" => true,
  "safari" => "Safari",
  "sis" => true,
  "sylera" => true,
  "w3clinemode" => true,
  "webcapture" => true,
  "webtv" => true,
  "w3m" => true,
  "webcrawler" => true,
  "wget" => true,
  "mozilla" => "Mozilla (or a compatible browser)");

/* Parses the $_SERVER[HTTP_USER_AGENT] string and returns an array where
     BrowserName => [Browser: MSIE, Firefox, Opera, Safari, etc. ]
     BrowserVersion => [ BrowserVersion ]
*/
function GetBrowser() {
  global $_AGENT_NAMES;
  /* Break the HTTP user agent string apart at certain punctuation characters
     and spaces
  */
  $userAgent = Tokenize(strtolower($_SERVER["HTTP_USER_AGENT"]), " /();");
  if (!$userAgent) return(NULL);
  // Reverse the array, since browser names/versions are usually toward the end
  $userAgent = array_reverse($userAgent);
  // This will hold the result of our efforts
  $result = array();
  // Find the browser identifier
  for ($i = 0; $i < count($userAgent); $i++) {
    // If the user agent was found...
    if (isset($_AGENT_NAMES[$userAgent[$i]])) {
      // Store the browser name (if one is specified)
      if (is_string($_AGENT_NAMES[$userAgent[$i]]))
        $result["Name"] = $_AGENT_NAMES[$userAgent[$i]];
      // Otherwise, use the agent identifier as the browser name
      else $result["Name"] = ucfirst($userAgent[$i]);
      // Store the user agent id
      $result["Agent"] = $userAgent[$i];
      break;
    }
  }
  // If a browser name was found, then retrieve the browser version
  if (isset($result["Name"])) $result["Version"] = $userAgent[$i-1];
  // Return what was found
  return($result);
}

/* This global variable will contain the browser, browser version, and user
   agent declaration */
$_BROWSER = GetBrowser();

/* FUNCTIONS FOR USE WITH SMARTY TEMPLATES ************************************/

// Generates an image tag
function ImageTag($params) {
  GLOBAL $_BROWSER;
  // This will hold our output
  $output = "";
  // Retrieve the image source
  if (!empty($params["src"])) {
    $imageSource = strtolower($params["src"]);
    // Get information about the image
    $imageInfo = getimagesize($imageSource);
    // If we failed to get the image information, quit
    if (!$imageInfo) return("");
    // Retrieve the image title, if one is specified
    if (!empty($params["title"])) $imageTitle = $params["title"];
    else if (!empty($params["alt"])) $imageTitle = $params["alt"];
    // Retrieve the image style, if one is specified
    if (!empty($params["style"])) $imageStyle = $params["style"];
    else $imageStyle = "";
    // Retrieve the width and height, if specified
    if (!empty($params["width"])) $imageStyle .= "width:".$params["width"]."px;";
    // If not specified, use the width we obtained
    else $imageStyle .= "width:".$imageInfo[0]."px;";
    if (!empty($params["height"])) $imageStyle .= "height:".$params["height"]."px;";
    // If not specified, use the height we obtained
    else $imageStyle .= "height:".$imageInfo[1]."px;";
    // Retrieve stylesheet information, if specified
    if (!empty($params["class"])) $imageClass = $params["class"];
    if (!empty($params["id"])) $imageID = $params["id"];
    // Check to see if the image is a PNG and MSIE is being used
    if (($imageInfo[2] == 3) && ($_BROWSER["Agent"] == "msie")) {
      // Generate the <span> tag
      $output = "<SPAN";
      // Include the stylesheet information, if specified
      if (!empty($imageClass)) $output .= " CLASS=\"".$imageClass."\"";
      if (!empty($imageID)) $output .= " ID=\"".$imageID."\"";
      // Include the image title, if specified
      if (!empty($imageTitle)) $output .= " TITLE=\"".$imageTitle."\"";
      // Include the stylistic elements of the span
      $output .= " STYLE=\"filter:progid:DXImageTransform.Microsoft.";
      $output .= "AlphaImageLoader(src='$imageSource', sizingMethod='scale');";
      // Add any other style elements included with the image
      if (!empty($imageStyle)) $output .= " ".$imageStyle;
      $output .= "\"></SPAN>";
      // Rturn the output
      return($output);
    }
    // Otherwise, output a normal image tage
    $output = "<IMG SRC=\"$imageSource\"";
    if (!empty($imageClass)) $output .= " CLASS=\"".$imageClass."\"";
    if (!empty($imageID)) $output .= " ID=\"".$imageID."\"";
    // Include the image title, if specified
    if (!empty($imageTitle)) $output .= " TITLE=\"".$imageTitle."\"";
    // Include the stylistic elements of the span
    if (!empty($imageStyle)) $output .= " STYLE=\"".$imageStyle."\"";
    $output .= ">";
  }
  return($output);
}

// Generates an anchor ("<a>") tag
function LinkTag($params) {
  // This will hold our output
  $output = "";
  // Change the array keys to lowercase (we don't care about case)
  $params = array_change_key_case($params);
  // Check to see if we have valid parameters
  if (empty($params["href"]) || empty($params["text"])) return ("");
  // Retrieve the link location
  $output = "<A HREF=\"".$params["href"]."\"";
  // Retrieve the link class, if one is specified
  if (!empty($params["id"])) $output .= " ID=\"".$params["id"]."\"";
  if (!empty($params["class"])) $output .= " CLASS=\"".$params["class"]."\"";
  // Retrieve any style information associated with the link
  if (!empty($params["style"])) $output .= " STYLE=\"".$params["style"]."\"";
  // Retreive the link target, if one is given
  if (!empty($params["target"])) $output .= " TARGET=\"".$params["target"]."\"";
  // Retrieve the link title, if one is specified
  if (!empty($params["title"])) {
    // Add the title
    $output .= " TITLE=\"".$params["title"]."\"";
    // Add the mouse events
    $output .= " onMouseOver=\"window.status='".$params["title"]."';return true\"";
    $output .= " onMouseOut=\"window.status='';return true\"";
  }
  $output .=">";
  // Include the link text
  $output .= $params["text"];
  // Close the tag
  $output .="</a>";
  // Return the output
  return($output);
}

// Test if a string or a single character in a string is uppercase
// if this returns false, then the "string" is lower case (by Indio)
function IsUpperCase($string, $location='-1') {

   // if a location is passed in, check that character only.
   // if not, or the passed in value is <0, check the whole
   // string for uppercase characters.

   if($location >= 0) {
         return(strcmp(strtolower($string{$location}),$string{$location}));
   }
   else {
         return(strcmp(strtolower($string),$string));
   }
}

// Returns the plural form of $word if $count is 0 or greater than 1
function Plural($word, $count) {
  // This array holds vowels
  $vowels = array(
    "a" => true,
    "e" => true,
    "i" => true,
    "o" => true,
    "u" => true);
  // This string will hold our output
  $output = "";
  // Check to see if we have valid parameters
  if (empty($word)) return ("");
  // Return the singular form of the word if $count is equal to one
  if ($count == 1) return($word);
  // If the word is "is" and there is more than one, return "are"
  $output = strtolower($word);
  if (($word == "is") && ($count != 1)) {
    if (IsUpperCase($word)) return("ARE");
    else return("are");
  }
  // Get the last character of the string
  $output = substr($word, -1, 1);
  // If the character is a y, the plural suffix is "ies"
  if ($output == "y") {
    $priorChar = strtolower(substr($word, -2, 1));
    if (!isset($vowels[$priorChar])) $output = substr($word, 0, -1)."ies";
    else $output = $word."s";
  }
  else $output = $word."s";
  // If the string passed in is all upper-case, return an all upper-case string
  if (IsUpperCase($word)) return(strtoupper($output));
  return($output);
}

// Returns an RFC 822 formatted timestamp from the specified date and time
function RFC822Timestamp($timestamp) {
  // Parse the timestamp into a Unix timestamp
  if (is_string($timestamp)) $timestamp = strtotime($timestamp);
  // Return the RFC822 timestamp
  return(date("r", $timestamp));
}

// Formats a timestamp using PHP's date() function (instead of strftime())
function Timestamp($timestamp, $format) {
  // Return the formatted timestamp
  if (is_string($timestamp)) $timestamp = strtotime($timestamp);
  return(date($format, $timestamp));
}


function gzdecode($data) {
  $len = strlen($data);
  if ($len < 18 || strcmp(substr($data,0,2),"\x1f\x8b")) {
   return null;  // Not GZIP format (See RFC 1952)
  }
  $method = ord(substr($data,2,1));  // Compression method
  $flags  = ord(substr($data,3,1));  // Flags
  if ($flags & 31 != $flags) {
   // Reserved bits are set -- NOT ALLOWED by RFC 1952
   return null;
  }
  // NOTE: $mtime may be negative (PHP integer limitations)
  $mtime = unpack("V", substr($data,4,4));
  $mtime = $mtime[1];
  $xfl  = substr($data,8,1);
  $os    = substr($data,8,1);
  $headerlen = 10;
  $extralen  = 0;
  $extra    = "";
  if ($flags & 4) {
   // 2-byte length prefixed EXTRA data in header
   if ($len - $headerlen - 2 < 8) {
     return false;    // Invalid format
   }
   $extralen = unpack("v",substr($data,8,2));
   $extralen = $extralen[1];
   if ($len - $headerlen - 2 - $extralen < 8) {
     return false;    // Invalid format
   }
   $extra = substr($data,10,$extralen);
   $headerlen += 2 + $extralen;
  }

  $filenamelen = 0;
  $filename = "";
  if ($flags & 8) {
   // C-style string file NAME data in header
   if ($len - $headerlen - 1 < 8) {
     return false;    // Invalid format
   }
   $filenamelen = strpos(substr($data,8+$extralen),chr(0));
   if ($filenamelen === false || $len - $headerlen - $filenamelen - 1 < 8) {
     return false;    // Invalid format
   }
   $filename = substr($data,$headerlen,$filenamelen);
   $headerlen += $filenamelen + 1;
  }

  $commentlen = 0;
  $comment = "";
  if ($flags & 16) {
   // C-style string COMMENT data in header
   if ($len - $headerlen - 1 < 8) {
     return false;    // Invalid format
   }
   $commentlen = strpos(substr($data,8+$extralen+$filenamelen),chr(0));
   if ($commentlen === false || $len - $headerlen - $commentlen - 1 < 8) {
     return false;    // Invalid header format
   }
   $comment = substr($data,$headerlen,$commentlen);
   $headerlen += $commentlen + 1;
  }

  $headercrc = "";
  if ($flags & 1) {
   // 2-bytes (lowest order) of CRC32 on header present
   if ($len - $headerlen - 2 < 8) {
     return false;    // Invalid format
   }
   $calccrc = crc32(substr($data,0,$headerlen)) & 0xffff;
   $headercrc = unpack("v", substr($data,$headerlen,2));
   $headercrc = $headercrc[1];
   if ($headercrc != $calccrc) {
     return false;    // Bad header CRC
   }
   $headerlen += 2;
  }

  // GZIP FOOTER - These be negative due to PHP's limitations
  $datacrc = unpack("V",substr($data,-8,4));
  $datacrc = $datacrc[1];
  $isize = unpack("V",substr($data,-4));
  $isize = $isize[1];

  // Perform the decompression:
  $bodylen = $len-$headerlen-8;
  if ($bodylen < 1) {
   // This should never happen - IMPLEMENTATION BUG!
   return null;
  }
  $body = substr($data,$headerlen,$bodylen);
  $data = "";
  if ($bodylen > 0) {
   switch ($method) {
     case 8:
       // Currently the only supported compression method:
       $data = gzinflate($body);
       break;
     default:
       // Unknown compression method
       return false;
   }
  } else {
   // I'm not sure if zero-byte body content is allowed.
   // Allow it for now...  Do nothing...
  }

  // Verifiy decompressed size and CRC32:
  // NOTE: This may fail with large data sizes depending on how
  //      PHP's integer limitations affect strlen() since $isize
  //      may be negative for large sizes.
  if ($isize != strlen($data) || crc32($data) != $datacrc) {
   // Bad format!  Length or CRC doesn't match!
   return false;
  }
  return $data;
} 

?>
