<?php

/* mysql.lib.php
   Provides an interface to a MySQL server instance.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("datasource.lib.php");
require_once("misc.lib.php");

// This class interfaces with a Microsoft SQL server instance
class MySQLDataSource extends DATA_SOURCE {
  // Constructor
  function MySQLDataSource($host, $database) {
    // Store the name of the database server
    $this->host = $host;
    // Store the name of the database to be accessed
    $this->dataSource = $database;
  }
  
  // Connects to the SQL server using the specified user name and password
  function Connect($userName, $password) {
    // Continue only if we were passed a valid-looking user name
    if (empty($userName)) return(seInvalidParam);
    // Store the username for future use
    $this->userName = $userName;
    // Try to open the database connection
    $this->handle = @mysql_pconnect($this->host, $this->userName, $password, 2);
    // If we failed, return an error
    if (!$this->handle) DiePrettily(seDataSource, array(
      get_class($this),
        " - The database backend was unable to obtain a response from the ",
        "database server: &lt;", $this->host, "&gt;."));
    DebugOut(get_class($this), array(
      "<strong>->Connect()</strong>",
      " successfully connected to: ", $this->host)); 
    // Try to open the database specified when the interface was created
    if (@mysql_select_db($this->dataSource, $this->handle)) {
      DebugOut(get_class($this), array(
        "<strong>->Connect()</strong>",
        " successfully opened database ", $this->dataSource));
      return(seOkay);
    }
    else DiePrettily(seDataUnavailable, array(
      get_class($this),
      " - The database backend was unable to open the database: ", 
      $this->dataSource));
  }
  
  // Executes the specified query string
  function Query($queryText) {
    // Ensure we have a valid-looking query string
    if (is_array($queryText)) $queryText = implode(" ", $queryText);
    if (empty($queryText)) return(seInvalidParam);
    // Stash the query string for future reference
    $this->queryString = $queryText.";";
    // Start timing the query
    $queryStart = microt(TRUE);
    // Send the query string to the server
    $this->queryResult = @mysql_query($this->queryString, $this->handle);
    // Stop timing the query
    $this->queryRunTime = round(microt(TRUE) - $queryStart, 4);
    // Output a debug message (if we're debugging)
    DebugOut(get_class($this), array(
      "<strong>->Query()</strong>",
      " finished in <strong>", $this->queryRunTime, "</strong> seconds ",
      "and returned ", ($this->queryResult ? $this->queryResult:"FALSE"), ".<br />",
      "The query was:<div style='padding-left:10px'>&lt;", $queryText, "&gt;.</div>",
      "The MySQL server replied with:<div style='padding-left:10px'>&lt;", 
      @mysql_error($this->handle), "&gt;</div>"));
    // If we failed, indicate an error
    if (!$this->queryResult) return(seQuery);
    // Update the query counter
    $this->queryCount++;
    // Return success
    return($this->queryResult);
  }
  
  // Retrieves the last message returned by the database server
  function Message() {
    // Return the last message
    return(@mysql_error($this->handle));
  }
  
  // Executes a SELECT query on the current database
  function Select($query) {
    // Continue only if we have a valid looking list of tables to search
    if (empty($query->location)) return(seInvalidParam);
    // Start constructing the query string
    $queryText = array("SELECT");
    // Add the list of fields to be retrieved from each table
    foreach($query->location as $tableName => $fieldList)
      // Add the field list directly to the string
      if (is_array($fieldList))
        $queryText[] = $tableName.".".implode(", ".$tableName.".", $fieldList);
      else $queryText[] = $fieldList;
    // Add the list of tables to the query string
    $queryText[] = "FROM ".implode(",", array_keys($query->location));
    // If filter criteria was specified for this search, include it
    if (isset($query->criteria) && !empty($query->criteria)) {
      $queryText[] = "WHERE";
      // If the criteria are specified in an array, get them as a string
      if (is_array($query->criteria))
        $queryText[] = implode("", $query->criteria);
      // Otherwise, add the criteria directly to the query string
      else $queryText[] = $query->criteria;
    }
    // If sort criteria was specified, include it
    if (isset($query->sort) && !empty($query->sort)) {
      $queryText[] = " ORDER BY ";
      if (is_array($query->sort)) {
        foreach($query->sort as $order => $fieldList) {
          // If more than one field is specified, retrieve them all
          if (is_array($fieldList))
            $queryText[] = implode(",", $fieldList)." ".$order;
          // Otherwise, add the field list directly to the query string
          else $queryText[] = $fieldList." ".$order;
        }
      }
      else $queryText[] = $query->sort;
    }
    // If an offset or limit was specified, include them
    if (!empty($query->offset)) {
      if (empty($query->limit)) $queryText[] = " LIMIT ".$query->offset.",18446744073709551615";
      else $queryText[] = " LIMIT ".$query->offset.",".$query->limit;
    }
    // Run the query and return the result
    $query->handle = $this->Query($queryText);
    $query->runTime = $this->queryRunTime;
    // Indicate the number of rows returned by the query
    $result = $this->RowCount($query);
    $query->resultCount = $result + $query->offset;
    return($result);
  }
  
  // Executes a DELETE or TRUNCATE TABLE query on the current database
  function Delete($query, $now = FALSE) {
    // Continue only if we have a valid-looking list of tables
    if (empty($query->location)) return(seInvalidParam);
    // If query criteria was specified, we are deleting one or more rows
    if (isset($query->criteria)) {
      // Build the query string
      $queryString = "DELETE ";
      /* LOW PRIORITY allows other users of the table to finish with it before
         the rows are deleted */
      if (!$now) $queryString .= "LOW_PRIORITY";
      // Add the list of tables to the query string
      $queryString .= " FROM ".implode(",", $query->location);
      // Add the query criteria to the query string
      $queryString .= " WHERE ";
      // If the criteria are specified in an array, get them as a string
      if (is_array($query->criteria))
        $queryString .= implode("", $query->criteria);
      // Otherwise, add the criteria directly to the query string
      else $queryString .= $query->criteria;
    }
    // Otherwise, if no criteria is specified, the entire table is to be deleted
    else $queryString = "TRUNCATE TABLE ".implode(",", $query->location);
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // Indicate the number of rows changed by this query
    return($this->RowsChanged($query));
  }
  
  // Executes a full-text search on the specified tables
  function FindText($query) {
    // Continue only if we have a valid-looking list of tables and criteria
    if (empty($query->location) || empty($query->criteria))
      return(seInvalidParam);
    // Build the query string
    $queryString = "SELECT ";
    $queryString .= implode(",", array_values($query->location));
    // Build the match string
    $match = " MATCH (";
    if (is_array($query->fieldsValues)) $match .= implode(",", $query->fieldsValues);
    else $match .= $query->fieldsValues;
    $match .= ") AGAINST ('";
    if (is_array($query->criteria)) $match .= Quote(implode("','", $query->criteria));
    else $match .= $query->criteria;
    $match .= "' IN BOOLEAN MODE)";
    // Add the "Relevance" column to the results
    $queryString .= ",".$match."*100 AS Relevance";
    // Add the list of tables to the query string
    $queryString .= " FROM ".implode(",", array_keys($query->location));
    // Add the query criteria
    $queryString .= " WHERE".$match;
    // If sort criteria was specified, include it
    if (isset($query->sort) && !empty($query->sort)) {
      $queryString .= " ORDER BY ";
      if (is_array($query->sort)) 
        foreach($query->sort as $order => $fieldList) {
          // If more than one field is specified, retrieve them all
          if (is_array($fieldList))
            $queryString .= implode(",", $fieldList)." ".$order;
          // Otherwise, add the field list directly to the query string
          else $queryString .= $fieldList." ".$order;
        }
      else $queryString .= $query->sort;
    }
    // If a limit or offset was specified, include it
    if ($query->limit > 0) $queryString .= " LIMIT ".$query->limit;
    if ($query->offset != 0) $queryString .= " OFFSET ".$query->offset;
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    $query->runTime = $this->queryRunTime;
    // Indicate the number of rows returned by the query
    $result = $this->RowCount($query);
    $query->resultCount = $result + $query->offset;
    return($result);
  }
  
  // Inserts data into the database
  function Insert($query) {
    /* Continue only if we have a valid-looking list of table names and fields/
       values */
    if (empty($query->location) || empty($query->fieldsValues)) 
      return(seInvalidParam);
    // There's no need to insert the primary key
    if (isset($query->fieldsValues["ID"])) {
      // Stash the primary key for a moment
      $primaryKey = $query->fieldsValues["ID"];
      // Remove it from the query
      unset($query->fieldsValues["ID"]);
    }
    // Build the query string
    $queryString = "INSERT INTO ";
    $queryString .= implode(",", $query->location);
    /* If the fields/values array is an associative array, we assume that we
       are setting values for specific fields in the table.  Otherwise, the
       fields/values array must contain one value for every field in the table,
       or the query will fail. */
    if (is_assoc($query->fieldsValues)) {
      $queryString .= " (";
      $queryString .= implode(",", array_keys($query->fieldsValues));
      $queryString .= ")";
    }
    // Add the values to be inserted
    $queryString .= " VALUES ('";
    $queryString .= implode("','", array_values($query->fieldsValues));
    $queryString .= "')";
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // If we had to unset the primary key, we restore it now
    if (isset($primaryKey)) $query->fieldsValues["ID"] = $primaryKey;
    // Indicate the number of rows changed by the query
    return($this->RowsChanged($query));
  }
  
  // Updates data in the specified fields for the specified tables
  function Update($query) {
    // Ensure that we have a valid list of tables, fields/values, and criteria
    if (empty($query->location) || empty($query->fieldsValues) ||
      empty($query->criteria)) return(seInvalidParam);
    // There's no need to update the primary key
    if (is_array($query->fieldsValues) && isset($query->fieldsValues["ID"])) {
      // Stash the primary key for a moment
      $primaryKey = $query->fieldsValues["ID"];
      // Remove it from the query
      unset($query->fieldsValues["ID"]);
    }
    // Build the query string
    $queryString = "UPDATE ";
    $queryString .= implode(",", $query->location);
    // Include the fields to be updated
    $queryString .= " SET ";
    if (is_array($query->fieldsValues)) { 
      foreach($query->fieldsValues as $fieldName => $fieldValue) {
        $queryString .= $fieldName."=\"".mysql_real_escape_string($fieldValue)."\",";
      }
      // Strip the trailing comma
      $queryString = rtrim($queryString, ",");
    }
    else $queryString .= $query->fieldsValues;
    // Include the criteria
    $queryString .= " WHERE ";
    // If the criteria are specified as an array, get them as a string
    if (is_array($query->criteria))
      $queryString .= implode("", $query->criteria);
    // Otherwise, add the criteria directly to the query string
    else $queryString .= $query->criteria;
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // If we had to unset the primary key, we restore it now
    if (isset($primaryKey)) $query->fieldsValues["ID"] = $primaryKey;
    // Return the number of rows affected by the query
    return($this->RowsChanged($query));
  }
  
  // Returns the last primary key generated by the database server
  function LastKey() {
    // Query for the last key generated
    $result = $this->Query("SELECT LAST_INSERT_ID()");
    // If we failed, indicate an error
    if ($result < 0) return($result);
    // Retrieve the row that contains the last primary key
    $resultRow = @mysql_fetch_row($result);
    // Return the key, if it was given to us by the server
    if (isset($resultRow[0])) return($resultRow[0]);
    else return(0);
  }
  
  // Retrieves the total rows returned by the specified query
  function RowCount($query) {
    // Ensure that the query handle is valid
    if (!$query->handle) return(seInvalidParam);
    // Retrieve the row count
    $rowCount = @mysql_num_rows($query->handle);
    // Output a debug message (if debugging)
    DebugOut(get_class($this), array(
      "<strong>->RowCount()</strong>",
      " found that ", $rowCount, " row(s) were returned."));
    // Return the row count
    return($rowCount);
  }
  
  // Retrieve the total number of rows changed by the specified query
  function RowsChanged() {
    // Retrieve the number of rows affected
    $rowCount = @mysql_affected_rows();
    // Output a debug message (if debugging)
    DebugOut(get_class($this), array(
      "<strong>->RowsChanged()</strong>",
      " found that ", $rowCount, " row(s) were affected."));
    // Return the row count
    return($rowCount);
  }
  
  // Retrieves the next row from the specified query
  function NextRow($query) {
    // Ensure the query handle is valid
    if (!$query->handle) return(FALSE);
    // Fetch the next row
    $queryRow = @mysql_fetch_row($query->handle);
    // Output a debug message (if debugging)
    DebugOut(get_class($this), array(
      "<strong>->NextRow()</strong>",
      " fetched row: <div style='padding-left:10px'>", 
      @implode("<br />", $queryRow), "</div>"));
    // Return the row
    return($queryRow);
  }
  
  /* Retrieves the next row from the specified query (both field names and
     values) */
  function NextRowVerbose($query) {
    // Ensure the query handle is valid
    if (!$query->handle) return(FALSE);
    // Fetch the next row
    $queryRow = @mysql_fetch_assoc($query->handle);
    // Output a debug message (if debugging)
    DebugOut(get_class($this), array(
      "<strong>->NextRowVerbose()</strong>",
      " fetched row: <div style='padding-left:10px'>", 
      implode_assoc($queryRow, "<br />","="),"'", "</div>"));
    // Return the row
    return($queryRow);
  }
  
  /* Frees a query from memory */
  function FreeQuery($query) {
    // Ensure the query handle is valid
    if (!$query->handle) return(seInvalidParam);
    return(@mysql_free_result($query->handle));
  }
  
  /* Retrieves the names of all of the tables in the database */
  function TableNames() {
    // Query on the table names
    $result = $this->Query("SHOW TABLES FROM ".$this->source);
    // If the query failed, return an error
    if ($result < 0) return($result);
    // Create an array to hold the table names
    $tableNames = array();
    // Retrieve each table name
    $queryRow = @mysql_fetch_row($result);
    while ($queryRow) {
      // Stash the table name
      $tableNames[] = $queryRow[0];
      // Get the next row
      $queryRow = @mssql_fetch_row($result);
    }
    // Return the list of table names
    return($tableNames);
  }
  
  /* Retrieves the names of the fields in the specified table */
  function FieldNames($tableName) {
    // Continue only if we have a valid-looking table name
    if (empty($tableName)) return(seInvalidParam);
    // Run a dummy query on the table
    $dummy = $this->Query("SELECT * FROM ".$tableName." LIMIT 1");
    // If the query failed, indicate an error
    if ($dummy < 0) return($dummy);
    // Retrieve the field names
    $dummyRow = @mysql_fetch_assoc($dummy);
    $fieldNames = array_keys($dummyRow);
    // Free the query
    @mysql_free_result($dummy);
    // Return the field names
    return($fieldNames);
  }
  
  /* Converts a PHP time value into a string that the database can handle.  If
     no time value is specified, the current time is used. */
  function TimeValue($time = -1) {
    if ($time < 0) $time = time();
    return(strftime("%Y-%m-%d %H:%M:%S", $time));
  }
}

?>
