<?php

/* post.lib.php
   Specifies an object that retrieves posts (data) from a Siteation table.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("query.lib.php");
require_once("misc.lib.php");

/* This class retrieves posts (one or more pieces of data) from a Siteation
   table for use in a Siteation view.  If no table name is given, the table that
   holds posts is assumed. */
class SITE_POST extends QUERY {
  // Stores the handle assigned to the query when it has executed
  public $handle;
  // Stores the data source used by the query
  public $dataSource;
  // Stores the list of tables and fields to search for
  public $location;
  // Stores the list of filter criteria used to modify the search
  public $criteria;
  // Stores the list of fields and values to be changed or added by the query
  public $fieldsValues;
  // Stores the list of fields on which to sort
  public $sort;
  // Stores the limit on the number of records returned by the query
  public $limit;
  /* Stores the number of matching records to skip before the query returns
     results */
  public $offset;
  // Stores the query execution time, in milliseconds
  public $runTime;
  // Stores the total number of results for the query
  public $resultCount;
  // Stores the most recent record retrieved by the query
  private $settings;
  /* Indicates whether or not the record has been changed since it was last
     read from or written to the database */
  private $changed;
  
  // Constructs a query instance for the specified data source
  function __construct($dataSource) {
    // Continue only if we have a valid data source
    if (!isset($dataSource)) DiePrettily(seInvalidParam, $this);
    global $SITE_CONFIG;
    // Store the data source for future reference
    $this->dataSource = $dataSource;
    $this->offset = 0;
  }
  
  // Destroys the query and frees any results associated with it from memory
  function __destruct() {
    if ($this->handle >= 0) $this->dataSource->FreeQuery($this);
  }
  
  /* Sets the table and that the post will read from or write to and the fields
     that it will return */
  function SetLocation($location) {
    // Continue only if we have a valid-looking location
    if (empty($location) || !is_array($location)) return(seInvalidParam);
    // Store the query location
    $this->location = $location;
  }
  
  /* Sets the criteria that will be used when reading or writing post data */
  function SetCriteria($criteria) {
    // Continue only if we have valid-looking criteria
    if (empty($criteria)) return(seInvalidParam);
    // Store the query criteria
    $this->criteria = $criteria;
  }
  
  // Executes a SELECT query with this query's information
  function Select() {
    // Execute the query
    return($this->dataSource->Select($this));
  }
  
  // Executes a DELETE or TRUNCATE TABLE query with this query's information
  function Delete($now = FALSE) {
    // Execute the query
    return($this->dataSource->Delete($this));
  }
  
  // Executes an INSERT query with this query's information
  function Insert() {
    // Execute the query
    return($this->dataSource->Insert($this));
  }
  
  // Executes an UPDATE query with this query's information
  function Update() {
    // Execute the query
    return($this->dataSource->Update($this));
  }
  
  // Executes a full-text search query with this query's information
  function FindText() {
    // Execute the query
    return($this->dataSource->FindText($this));
  }
  
  // Retrieves the number of rows returned by this query
  function Count() {
    GLOBAL $SITE_CONFIG;
    // Retrieve the row count (for non MSSQL datasets)
    if ($SITE_CONFIG["SQL_SERVER"] != "MSSQL")
      return($this->dataSource->RowCount($this));
    return($this->dataSource->RowCount($this) - $this->offset);
  }
  
  // Retrieves the number of rows changed by this query
  function RowsChanged() {
    // Retrieve the row count
    return($this->dataSource->RowsChanged($this));
  }
  
  // Retrieves the next row for this query
  function NextRow() {
    // Retrieve the next row
    return($this->dataSource->NextRow($this));
  }
  
  // Retrieves the query run time
  function RunTime() {
    // Return the query run time
    return($this->runTime);
  }
  
  function ResultCount() {
    // Return the query result count
    return($this->resultCount);
  }
  
  // Retrieves the next row (including field names and values) for this query
  function Next() {
    GLOBAL $SITE_CONFIG;
    // Retrieve the next row
    $result = $this->dataSource->NextRowVerbose($this);
    // If we were unsuccessful, return the result
    if ($result <= 0) return($result);
    // Check to see if this post has extra fields (variables)
    if (isset($result["Extra"]) && !empty($result["Extra"]))
      // Expand the variables
      $result["Extra"] = explode_assoc($result["Extra"], "|");
    // Determine how many replies this post has
    /*
    if (array_key_exists([$SITE_CONFIG["TABLE_PREFIX"]."posts", $this->location)) {
      $replies = new QUERY($this->dataSource);
      // Set the query location
      $replies->location = array(
        $SITE_CONFIG["TABLE_PREFIX"]."posts" => "ID");
      // Set the query criteria
      $replies->criteria = array(
        "Parent",        " = ", $result["ID"], " OR ",
        "ThreadParent",  " = ", $result["ID"]);
      // Run the query
      $outcome = $replies->Select();
    if ($outcome >= 0) $result["Replies"] = $outcome;
    // Free the query
    $replies = null;
    */
    return($result);
  }
}

?>
