<?php

/* query.lib.php
   Specifies a class that interfaces with a DATA_SOURCE object to execute
   queries.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("misc.lib.php");

// This class stores and executes queries
class QUERY {
  // Stores the handle assigned to the query when it has executed
  public $handle;
  // Stores the data source used by the query
  public $dataSource;
  // Stores the list of tables and fields to search for
  public $location;
  // Stores the list of filter criteria used to modify the search
  public $criteria;
  // Stores the list of fields and values to be changed or added by the query
  public $fieldsValues;
  // Stores the list of fields on which to sort
  public $sort;
  // Stores the limit on the number of records returned by the query
  public $limit;
  /* Stores the number of matching records to skip before the query returns
     results */
  public $offset;
  // Stores the query execution time, in milliseconds
  public $runTime;
  // Stores the number of query results
  public $resultCount;
  // Stores the query string constructed from the query
  public $queryString;
  
  // Constructs a query instance for the specified data source
  function __construct($dataSource) {
    // Continue only if we have a valid data source
    if (!isset($dataSource)) DiePrettily(seInvalidParam, $this);
    // Store the data source for future reference
    $this->dataSource = $dataSource;
    $this->offset = 0;
  }
  
  // Destroys the query and frees any results associated with it from memory
  function __destruct() {
    if ($this->handle >= 0) $this->dataSource->FreeQuery($this);
  }
  
  // Retrieves the last message returned by the database server
  function Message() {
    return($this->dataSource->Message());
  }
  
  // Executes a direct query with the specified query string
  function DirectQuery($queryString) {
    // Execute the query
    $this->handle = $this->dataSource->Query($queryString);
    // Return the result
    return($this->handle);
  }
  
  // Executes a SELECT query with this query's information
  function Select() {
    // Execute the query
    return($this->dataSource->Select($this));
  }
  
  // Executes a DELETE or TRUNCATE TABLE query with this query's information
  function Delete($now = FALSE) {
    // Execute the query
    return($this->dataSource->Delete($this));
  }
  
  // Executes an INSERT query with this query's information
  function Insert() {
    // Execute the query
    return($this->dataSource->Insert($this));
  }
  
  // Executes an UPDATE query with this query's information
  function Update() {
    // Execute the query
    return($this->dataSource->Update($this));
  }
  
  // Executes a full-text search query with this query's information
  function FindText() {
    // Execute the query
    return($this->dataSource->FindText($this));
  }
  
  // Retrieves the number of rows returned by this query
  function RowCount() {
    // Retrieve the row count
    return($this->dataSource->RowCount($this));
  }
  
  // Retrieves the number of rows changed by this query
  function RowsChanged() {
    // Retrieve the row count
    return($this->dataSource->RowsChanged($this));
  }
  
  // Retrieves the next row for this query
  function NextRow() {
    // Retrieve the next row
    return($this->dataSource->NextRow($this));
  }
  
  // Retrieves the next row (including field names and values) for this query
  function NextRowVerbose() {
    // Retrieve the next row
    return($this->dataSource->NextRowVerbose($this));
  }
  
  // Retrieves the query run time
  function RunTime() {
    // Return the query run time
    return($this->runTime);
  }
  
  // Retrieves the TOTAL number of results for the query
  function ResultCount() {
    return($this->resultCount);
  }
}

?>
