<?php

/* This file defines several modifiers and functions that can be used in view 
   templates.
*/

/* Retrieves view data (posts) 
   Usage (in-template):
     [ fetching from="<table>" fields="<fieldnames>" filter="<criteria>" /
         offset="<offset>" limit="<limit>" sort="<sort order>"  /
         into="<name of template variable>" ]
         
   Note that only the "from" and "into" parameters are required; the rest
   are optional.  If "fields" is omitted, all fields are returned for each
   row of the query.
*/
function viewFetchData($params, &$view) {
    global $SITE_CONFIG;
    // Continue only if this view is valid (we have retrieved view information)
    if (empty($params)) return;
    $results = new SITE_POST($view->DataSource());
    if (!$results) return;
    // Get the table to be searched
    if (!empty($params["from"])) {
      $results->location = array($SITE_CONFIG["TABLE_PREFIX"].$params["from"] => "*");
      if (!empty($params["fields"]))
        $results->location[$params["from"]] = $params["fields"];
    }
    if (!empty($params["filter"])) 
      $results->criteria = $params["filter"];
    // Store the view limits, if given
    if (!empty($params["limit"]))
      $results->limit = $params["limit"];
    if (!empty($params["offset"]))
      $results->offset = $params["offset"];
    // Store the sort criteria, if given
    if (!empty($params["sort"]))
      $results->sort = $params["sort"];
    // Run the query
    $results->Select();
    $view->assign($params["into"], $results);
  }

/* This function retrieves the next row from the query (previously obtained
     by using the "fetching" template function).
   Usage (in-template):
     [ fetching from="posts" filter="ChannelName='Home'" into=posts ]
     [ $posts|next ]
*/
function viewNextRowFrom(SITE_POST $source) {
  if ($source) return ($source->Next());
}

/* This function retrieves the total number of results in the query (previously
     obtained by using the "fetching" template function).
   Usage (in-template):
    [ fetching from="posts" filter="ChannelName='Home'" into=posts ]
    [ if $posts|count gt 0 ]
     ...
    [ /if ]
*/
function viewCountResultsIn(SITE_POST $source) {
  if ($source) return ($source->Count());
}

/* This function retrieves the total runtime required to produce the query
     (previously obtained by using the "fetching" template function).
   Usage (in-template):
    [ fetching from="posts" filter="ChannelName='Home'" into=posts ]
    Search took <strong>[ $posts|runtime ]</strong> seconds.
*/
function viewRuntimeFor(SITE_POST $source) {
  if ($source) return ($source->RunTime());
}

/* This function frees a query previously obtained using the "fetching"
     template function.
   Usage (in-template):
    [ fetching from="posts" filter="ChannelName='Home'" into=posts ]
      ...
    [ $posts|done ]
*/
function viewDone(SITE_POST $source) {
  if ($source) $source->__destruct();
}

?>
