/* This script organizes the entire list of channels in a Siteation
   database.  Part of Siteation.  Copyright (C) 2005 Studio 703.
   All rights reserved.
*/

// These settings determine which icons to use when a section of the tree is
// expanded or collapsed
var TreeOpen = "/images/site/list.collapse.png";
var TreeClosed = "/images/site/list.expand.png";

function getActiveLeaf() {
  // Retrieve the top of the list
  channelList = top.document.getElementById("Channels");
  if (!channelList) return null;
  child = channelList.firstChild;
  while (child) {
    if (child.className == "active") return(child);
    child = child.nextSibling;
  }
  return(null);
}

// This function handles mouse clicks for individual leaves
function PickLeaf(leafName) {
  // Retrieve the currently-selected leaf anchor
  var currentLeaf = getActiveLeaf();
  // Reset the leaf class name
  if (currentLeaf) currentLeaf.className = "";
  // Highlight the new leaf
  leaf = top.document.getElementById(leafName);
  if (leaf) {
    leaf.className = "active";
    // Make sure the current channel always shows
    leaf = leaf.parentNode;
    // Work backwards
    while (leaf.id != "Channels") {
      // Ensure the parent node (if it's a <li> element) and its associated children
      // are visible
      if ((leaf.tagName == "LI") && (leaf.style.display == "none")) 
        showHide(leaf);
      // Get the next node
      leaf = leaf.parentNode;
    }
  }
  // If the leaf does not yet exist, set the channel name for use when it does
  else {
    channel = top.document.getElementById("ChannelName");
    return;
  }
  return true;
}

// This function inserts a leaf for the specified parent
function SproutLeaf(branch, leafName, icon) {
  // Correct for the root node
  if (leafName == "/") result = document.getElementById("Channels");
  // See if the leaf already exists
  else result = document.getElementById(leafName);
  // If the leaf already exists, great!
  if (result) return(result);
  // Correct for the root node as parent
  if (branch == "/") branch = document.getElementById("Channels");
  // Otherwise, create the leaf
  newLeaf = document.createElement("LI");
  newLeaf.id = leafName;
  newLeaf.onmouseover = function() { this.style.border = "dashed 1px #6ec6fn"; return true; }
  newLeaf.onmouseout = function() { this.style.border = "none"; return true; }
  // Create the leaf icon
  newLeafIcon = document.createElement("IMG");
  newLeafIcon.id = leafName + "_ICON";
  // Set the class name of the new icon
  newLeafIcon.className = "icon";
  newLeafIcon.style.width = "25px";
  newLeafIcon.style.height = "25px";
  // Set the icon image
  if (icon != "") newLeafIcon.src = icon;
  else newLeafIcon.src = TreeItem;
  // Attach the icon to the new leaf
  newLeaf.appendChild(newLeafIcon);
  // Create the leaf link
  newLeafLink = document.createElement("A");
  newLeafLink.href = "viewpoint.php?channel=" + encodeURIComponent(leafName);
  newLeafLink.title = leafName;
  //newLeafLink.target = "main";
  //newLeafLink.onclick = function() { return PickLeaf(this.title); }
  newLeafLink.onmouseover = function() { window.status='Click to view "' + leafName + '".'; return true; }
  newLeafLink.onmouseout = function() { window.status=''; return true; }
  newLeafLink.style.border = "none";
  // Insert the link
  newLeaf.appendChild(newLeafLink);
  // Create a text node that contains the channel name
  newLeafText = document.createTextNode(leafName);
  newLeafLink.appendChild(newLeafText);
  // If we're at the top of the channel list, then the leafParent is "Channels"
  if (branch.id == "Channels") leafParent = branch;
  // Otherwise, if the branch has no children as yet, it must become a parent
  else if (branch.getElementsByTagName("ul").length == 0) {
    // Create a new unordered list
    leafParent = document.createElement("UL");
    leafParent.id = branch.id + "_UL";
    // Hide the new unordered list for now
    //leafParent.style.display = "none";
    // Insert the unordered list underneath the parent
    branch.appendChild(leafParent);
    // Set the icon to represent the new parent
    parentIcon = document.getElementById(branch.id + "_ICON");
    // parentIcon.src = TreeOpen;
    parentIcon.title = "Collapse this list"
    // Arrange to have clicks expand or collapse the tree
    parentIcon.onclick = function() { showHide(this.parentNode); return true; }
  }
  else {
    // We need to find the <ul> element that will act as the parent for our
    // new <li> element
    if (branch.id != "Channels")
      leafParent = document.getElementById(branch.id + "_UL");
  }
  // Sprout a new leaf right on the branch
  leafParent.appendChild(newLeaf);
  // Return the leaf
  return(newLeaf);
}

// Shows or hides a channel list element
function showHide(what) {
  // Get the element
  element = document.getElementById(what.id + "_UL");
  // Show or hide the element
  element.style.display = (element.style.display != "none") ? "none":"block";
  // Change the icon that matches the element to show the list is expanded or collapsed
  icon = document.getElementById(what.id + "_ICON");
  icon.title = (element.style.display == "none") ? "Expand this list":"Collapse this list";
  //icon.src = (element.style.display != "none") ? TreeOpen:TreeClosed;
}

// Clears the channel tree
function clearTree() {
  // See if the channel list exists
  channelTree = document.getElementById("Channels");
  // If it does, see if it has any children
  if (channelTree) {
    leaf = channelTree.firstChild;
    // Remove all children from the tree
    while(leaf) {
      channelTree.removeChild(leaf);
      leaf = channelTree.firstChild;
    }
  }
}

// @Parent|ChannelName|ChannelIcon
function buildTree(data) {
  // Proceed only if we have data
  if (!data) return;
  if (data == "") return;
  // Clear the channel tree
  clearTree();
  // Break out the channel records
  channels = data.split("@");
  // Break out the channel fields
  for (var i=1; i < channels.length; i++) {
    channels[i] = channels[i].split("|");
  }
  // Parse the channel records
  for (i = 1; i < channels.length; i++) {
    parentLeaf = document.getElementById(channels[i][0]);
    if (!parentLeaf) parentLeaf = document.getElementById("Channels");
    parentLeaf = SproutLeaf(parentLeaf, channels[i][1], channels[i][2]);
  }
  channel = document.getElementById("ChannelName");
  if (channel) PickLeaf(channel.firstChild.nodeValue);
  // Hide the "loading" message
  toggleHidden("navStatus");
}

// Builds the channel tree
function buildChannelTree() {
  // Clear the channel list
  clearTree();
  // Display a "loading" message
  toggleHidden("navStatus");
  // Create an HTTP request to retrieve the channel list
  remote = new HTTPRequest("GET", "text/plain");
  // If we failed to create the request, we must do things the old-fashioned way
  if (!remote) return;
  remote.url = "/scripts/webriot/viewpoint.php";
  remote.queryString = "view=Channel.List";
  remote.callback = function () { 
    // Check the server response
    if (remote.request.readyState == 4) {
      // Everything went well
      if (remote.request.status == 200) 
        buildTree(remote.request.responseText);
    }
  }
  // Send the request
  remote.send();
}  
