/* This script and many more are available free online at
   The JavaScript Source :: http://javascript.internet.com
   Created by: Michael O'Connell :: http://wunder-ful.com
   Modified for my own heinous purposes by Michael Malicoat :: http://www.studio703.com
*/

/**
* @return the sum of all the offsetTop values from this element to the body
*/
function getTotalOffsetTop(element) {
  var offset = 0;
  while (element) {
    if (element.offsetTop) offset += element.offsetTop;
    element = element.parentNode;
  }
  return offset;
}

/**
* @return the sum of all the offsetLeft values from this element to the body
*/
function getTotalOffsetLeft(element) {
  var offset = 0;
  while (element) {
    if (element.offsetLeft) offset += element.offsetLeft;
    element = element.parentNode;
  }
  return offset;
}

/**
* Positions a absolutely positioned element over the anchor.  It will detect body clipping
* and move the right side of the element to the right of the anchor.  The element cannot
* have
* @param element the element to position
* @param anchor the anchor to position element over
* @param positionVertical position on "top" or "bottom" edge of the anchor
* @param positionHorizontal position on "left" or "right" edge of the anchor
*/
function positionAtAnchor(element, anchor, positionVertical, positionHorizontal) {
  var left = getTotalOffsetLeft(anchor);

  if(positionHorizontal == "right")
    left = left + anchor.offsetWidth + 1;

  //horizontal clip
  if(left + element.offsetWidth > document.body.scrollWidth && left - element.offsetWidth > 0) {
    left = left - element.offsetWidth;
    if(positionHorizontal == "right")
      left = left - anchor.offsetWidth;
    else
      left = left + anchor.offsetWidth + 1;
  }

  var top = getTotalOffsetTop(anchor);
  if(positionVertical == "bottom")
    top = top + anchor.offsetHeight + 1;

  //vertical clip
  if(top + element.offsetHeight > document.body.scrollHeight && top - element.offsetHeight > 0) {
    top = top - element.offsetHeight;
    if(positionVertical == "bottom")
      top = top - anchor.offsetHeight;
    else
      top = top + anchor.offsetHeight + 1;
  }

  element.style.top = top+"px";
  element.style.left = left+"px";
}


// ===========================

//map of dynaCalendars by name
var dynaCalendars = {};

/**
* Opens / closes the dynacalendar at the specified anchor
* @param name The unique name for the calendar to open.
* @param anchor A element to open the calendar under
**/
function openDynaCalendar(name, anchor) {
  var calendar = dynaCalendars[name];
  if(!calendar.isOpen)
    calendar.open(anchor);
  else
    calendar.close();
}

/**
* Constructor for the DynaCalendar
* @param name A unique name for this calendar
* @param callback Callback function invoked when a date is clicked.
* @param year Year to start calendar.
* @param month Month to start calendar.
* @param day Day to start calendar.
* @see Invocation
**/
function DynaCalendar(name, callback, year, month, day) {
  //populate day of week and month name constants if not previously created...
  if(!DynaCalendar.MONTHS) {
    var MONTHS = [];
    var DOW = [];
    DynaCalendar.MONTHS = MONTHS;
    DynaCalendar.DOW = DOW;

    var tmpDate = new Date(2000, 0, 1);
    for(var i = 0; i < 12; i++) {
      tmpDate.setMonth(i);
      var dateStr = tmpDate.toLocaleString();
      var monthname = dateStr.match(/\s[a-zA-Z]+\s/)[0];
      monthname = monthname.substring(1,monthname.length-1);
      MONTHS[i] = monthname;
    }

    for(var i = 1; i < 8; i++) {
      tmpDate.setDate(i);
      var dateStr = tmpDate.toLocaleString();
      var dow = dateStr.substring(0,3);
      DOW[tmpDate.getDay()] = dow;
    }
  }

  if(year && month && day)
    this.date = new Date(year, month-1, day);
  else
    this.date = new Date();

  this.callback = callback;
  this.name = name;
  this.isOpen = false;
  this.rendered = false;
  this.bodyRendern = false;

  dynaCalendars[name] = this;
}

/**
* Displays the dynacalendar at the specified anchor.
* @param anchor A element to open the calendar under
*/
DynaCalendar.prototype.open = function() {
  if(!this.rendered)
    this.render();
  if(!this.bodyRendern)
    this.renderBody();
  this.calendarDiv.style.display = "block";
  // positionAtAnchor(this.calendarDiv, anchor, "bottom", "left");
  this.isOpen = true;
}

/**
* Closes the dynaCalendar
*/
DynaCalendar.prototype.close = function() {
  this.calendarDiv.style.display = "none";
  this.isOpen = false;
}

/**
* Fills the calendarDiv with actual date spans
**/
DynaCalendar.prototype.render = function() {
  var calendarDiv = document.createElement("DIV");
  var calendarTable = document.createElement("TABLE");
  var calendarHead = document.createElement("THEAD");
  var calendarHeadTR = document.createElement("TR");
  var calendarHeadWeekTR = document.createElement("TR");
  var calendarBY = document.createElement("TH");
  var calendarBM = document.createElement("TH");
  var calendarTitle = document.createElement("TH");
  var calendarFM = document.createElement("TH");
  var calendarFY = document.createElement("TH");
  var calendarBody = document.createElement("TBODY");

  calendarDiv.className = "dynaCalendarDiv";
  calendarTable.className = "dynaCalendarTable";
  calendarTitle.colSpan = 3;

  calendarBY.className = "BF";
  calendarBM.className = "BF";
  calendarFM.className = "BF";
  calendarFY.className = "BF";
  calendarBY.innerHTML = "&#9650;";
  calendarBM.innerHTML = "&#9668;";
  calendarFM.innerHTML = "&#9658;";
  calendarFY.innerHTML = "&#9660;";
  calendarBY.onclick = dynaCalendarBackYearOnClick;
  calendarBM.onclick = dynaCalendarBackMonthOnClick;
  calendarFM.onclick = dynaCalendarForwardMonthOnClick;
  calendarFY.onclick = dynaCalendarForwardYearOnClick;
  
  calendarBY.title = "Previous Year";
  calendarBM.title = "Previous Month";
  calendarFM.title = "Next Month";
  calendarFY.title = "Next Year";

  calendarDiv.dynaCalendar = this;
  calendarHead.dynaCalendar = this;
  calendarBody.dynaCalendar = this;
  calendarBY.dynaCalendar = this;
  calendarBM.dynaCalendar = this;
  calendarTitle.dynaCalendar = this;
  calendarFM.dynaCalendar = this;
  calendarFY.dynaCalendar = this;

  this.calendarTitle = calendarTitle;
  this.calendarBody = calendarBody;
  this.calendarDiv = calendarDiv;

  //populate week header
  for(var i = 0; i < 7; i++) {
    var dowTH = document.createElement("TH");
    dowTH.innerHTML = DynaCalendar.DOW[i];
    calendarHeadWeekTR.appendChild(dowTH);
  }

  calendarHeadTR.appendChild(calendarBY);
  calendarHeadTR.appendChild(calendarBM);
  calendarHeadTR.appendChild(calendarTitle);
  calendarHeadTR.appendChild(calendarFM);
  calendarHeadTR.appendChild(calendarFY);
  calendarHead.appendChild(calendarHeadTR);
  calendarHead.appendChild(calendarHeadWeekTR);
  calendarTable.appendChild(calendarHead);
  calendarTable.appendChild(calendarBody);
  calendarDiv.appendChild(calendarTable);
  document.body.appendChild(calendarDiv);

  this.rendered = true;
}

/* Hides or shows all floaters EXCEPT the one specified */
function displayOtherFloaters(floater, visibility) {
  // Get the first div in the document
  var elements = top.document.getElementsByTagName("DIV");
  // Loop through the list and set all visiblities EXCEPT for the floater specified
  for (var i=0; i < elements.length; i++) {
    if ((elements[i].id != floater.id) && (elements[i].id.indexOf("_float") >= 0)) elements[i].style.display = visibility;
  }
}

function floatersVisible() {
  // Get the first div in the top document
  var divs = top.document.getElementsByTagName("DIV");
  // Loop through the list looking for visible floaters
  for (var i=0; i < divs.length; i++) {
    if ((divs[i].className == "floater") && (divs[i].style.display != "none")) return divs[i];
  }
  return null;
}

/* Marks a date as important */
DynaCalendar.prototype.markDate = function(date, text) {
  // Retrieve the date in question
  var thisDate = document.getElementById(DynaCalendar.MONTHS[this.date.getMonth()] + date);
  if (thisDate) {
    thisDate.className = "dayMarked";
    thisDate.onclick = dynaCalendarSelectDayOnClick;
    // Do we already have a floater for this date?
    var floater = top.document.getElementById(thisDate.id + "_float");
    // If not, create one
    if (!floater) {
      floater = top.document.createElement("DIV");
      floater.className = "floater";
      floater.id = thisDate.id + "_float";
      floater.style.display = "none";
      floater.innerHTML = "<div style=\"text-align:center\"><strong>" + date + " " + DynaCalendar.MONTHS[this.date.getMonth()] + ", " + this.date.getFullYear() + "</strong></div>";
      top.document.body.appendChild(floater);
      thisDate.onmouseover = function(mouse) {
        // If any floaters are visible, we must wait for them to close before we display ours
        if (floatersVisible()) return true;
        thisFloater = top.document.getElementById(this.id + "_float");
        if (!thisFloater) return true;
        // Display the floater
        thisFloater.style.display = "block";
        thisFloater.style.top = (window.frameElement.offsetTop + mouse.clientY) - 5;
        thisFloater.style.left = (window.frameElement.offsetLeft + mouse.clientX) - 5;
        // Resize the floater to fit its contents
        var element = thisFloater.lastChild;
        while ((element) && (element != thisFloater)) {
          if (thisFloater.style.width < element.style.width) thisFloater.style.width = element.style.width + 4;
          element = element.parent;
        }
        return true;
      }
      floater.onmouseover = function() {
        if (this.style.display != "block") this.style.display = "block";
        return true;
      }
      floater.onmouseout = function () {
        this.style.display = "none";
        return true;
      }
    }
    // Append to the floater contents
    floater.innerHTML += text;
  }
}

/* Compares two dates
   Returns < 0 if date1 is older (further into the past) than date 2
             0 if the two dates match
           > 0 if date1 is newer than date 2
*/
function CompareDates(date1, date2) {
  // Determine if the years match
  if (date1.getFullYear() != date2.getFullYear())
    return(date1.getFullYear() - date2.getFullYear());
  // Determine if the months match
  if (date1.getMonth() != date2.getMonth())
    return(date1.getMonth() - date2.getMonth());
  // Determine if the days match
  if (date1.getDate() != date2.getDate())
    return(date1.getDate() - date2.getDate());
  // If we make it this far, the two dates match
  return(0);
}

/**
* Renders the actual month.
**/
DynaCalendar.prototype.renderBody = function() {
  //title
  var title = DynaCalendar.MONTHS[this.date.getMonth()]+ " " + this.date.getFullYear();
  this.calendarTitle.innerHTML = title;

  //clear table body
  while(this.calendarBody.hasChildNodes())
    this.calendarBody.removeChild(this.calendarBody.lastChild);

  //generate new table body
  var tmpDate = new Date(this.date.getFullYear(), this.date.getMonth(), 1);
  var today = new Date();

  var row = document.createElement("TR");
  this.calendarBody.appendChild(row);

  //blanks
  for(var i = 0; i < tmpDate.getDay(); i++) {
    var td = document.createElement("TD");
    row.appendChild(td);
  }

  while(tmpDate.getMonth() == this.date.getMonth()) {
    var td = document.createElement("TD");
    var date = tmpDate.getDate();
    if(CompareDates(tmpDate, today) == 0) {
      td.className = "daySelected";
      td.title = "Today"
    }
    else
      td.className = "day";

    td.innerHTML = date;
    td.dynaCalendar = this;
    td.id = DynaCalendar.MONTHS[this.date.getMonth()] + date.toString();
    // td.onclick = dynaCalendarSelectDayOnClick;
    row.appendChild(td);

    if(tmpDate.getDay() == 6) {
      row = document.createElement("TR");
      this.calendarBody.appendChild(row);
    }

    tmpDate.setDate(date + 1);
  }

  //blanks
  for(var i = tmpDate.getDay(); i < 7; i++) {
    var td = document.createElement("TD");
    row.appendChild(td);
  }

  this.bodyRendern = true;
}

/**
* Set the day of the month
* @param date The date of the month to set the calandar to.
**/
DynaCalendar.prototype.setDate = function(date) {
  this.date.setDate(date);
  this.bodyRendern = false;
}

/**
* Moves back a year
**/
DynaCalendar.prototype.backYear = function() {
  this.setDate(1);
  this.date.setFullYear(this.date.getFullYear() - 1);
  newYear = this.date.getFullYear();
  newMonth = this.date.getMonth() + 1;
  document.location.href="viewpoint.php?view=@recentUpdates.widget&dynamic=y&search=Parent=0+AND+MONTH(PubDate)=" + (newMonth) + "+AND+YEAR(PubDate)=" + newYear + "&ruwmonth=" + newMonth + "&ruwyear=" + newYear;
  //this.renderBody();
}

/**
* Moves back a month
**/
DynaCalendar.prototype.backMonth = function() {
  this.setDate(1);
  this.date.setMonth(this.date.getMonth() - 1);
  newYear = this.date.getFullYear();
  newMonth = this.date.getMonth() + 1;
  document.location.href="viewpoint.php?view=@recentUpdates.widget&dynamic=y&search=Parent=0+AND+MONTH(PubDate)=" + (newMonth) + "+AND+YEAR(PubDate)=" + newYear + "&ruwmonth=" + newMonth + "&ruwyear=" + newYear;
  //this.renderBody();
}

/**
* Moves forward a month
**/
DynaCalendar.prototype.forwardMonth = function() {
  this.setDate(1);
  this.date.setMonth(this.date.getMonth() + 1);
  newYear = this.date.getFullYear();
  newMonth = this.date.getMonth() + 1;
  document.location.href="viewpoint.php?view=@recentUpdates.widget&dynamic=y&search=Parent=0+AND+MONTH(PubDate)=" + (newMonth) + "+AND+YEAR(PubDate)=" + newYear + "&ruwmonth=" + newMonth + "&ruwyear=" + newYear;
  //this.renderBody();
}

/**
* Moves forward a year
**/
DynaCalendar.prototype.forwardYear = function() {
  this.setDate(1);
  this.date.setFullYear(this.date.getFullYear() + 1);
  newYear = this.date.getFullYear();
  newMonth = this.date.getMonth() + 1;
  document.location.href="viewpoint.php?view=@recentUpdates.widget&dynamic=y&search=Parent=0+AND+MONTH(PubDate)=" + (newMonth) + "+AND+YEAR(PubDate)=" + newYear + "&ruwmonth=" + newMonth + "&ruwyear=" + newYear;
  //this.renderBody();
}

// Event handlers -------------------------------------------------------------
/**
* onclick to select a day
*/
function dynaCalendarSelectDayOnClick(event) {
  //IE
  if(window.event) {
    var srcElement = window.event.srcElement;
  }
  //MOZ
  else {
    var srcElement = event.currentTarget;
  }

  var calendar = srcElement.dynaCalendar;
  calendar.setDate(srcElement.innerHTML);
  if(calendar.callback)
    calendar.callback(calendar.date);
    // calendar.invocation.invoke([calendar.date]);
  // srcElement.dynaCalendar.close();

  return false;
}

/**
* onclick to go back one year
*/
function dynaCalendarBackYearOnClick(event) {
  //IE
  if(window.event) {
    var srcElement = window.event.srcElement;
  }
  //MOZ
  else {
    var srcElement = event.currentTarget;
  }
  srcElement.dynaCalendar.backYear();
  return false;
}

/**
* onclick to go back one month
*/
function dynaCalendarBackMonthOnClick(event) {
  //IE
  if(window.event) {
    var srcElement = window.event.srcElement;
  }
  //MOZ
  else {
    var srcElement = event.currentTarget;
  }

  srcElement.dynaCalendar.backMonth();
  return false;
}

/**
* onclick to go forward one month
*/
function dynaCalendarForwardMonthOnClick(event) {
  //IE
  if(window.event) {
    var srcElement = window.event.srcElement;
  }
  //MOZ
  else {
    var srcElement = event.currentTarget;
  }

  srcElement.dynaCalendar.forwardMonth();
  return false;
}

/**
* onclick to go forward one year
*/
function dynaCalendarForwardYearOnClick(event) {
  //IE
  if(window.event) {
    var srcElement = window.event.srcElement;
  }
  //MOZ
  else {
    var srcElement = event.currentTarget;
  }

  srcElement.dynaCalendar.forwardYear();
  return false;
}
