<?php

/* MAIN ***********************************************************************/

// Buffer output
ob_start();

/* This code prevents client browsers and intermediate proxies from caching
   the output of this script.
*/
// Set the expiration date to a date in the past
header("Expires: ".gmdate("D, d M Y H:i:s"). " GMT");
// Indicate that the content is always modified
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
// HTTP/1.1 headers that will prevent buffering
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
// HTTP/1.0 headers that will prevent buffering
header("Pragma: no-cache");

/* viewpoint.php
   Defines the Siteation ViewPoint, which generates all views seen by website
   visitors.
   Part of Siteation.  Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.config.php");
require_once("libs/site.error.php");
require_once("libs/site.lib.php");
require_once("libs/session.lib.php");
require_once("libs/channel.lib.php");
require_once("libs/user.lib.php");
require_once("libs/view.lib.php");
require_once("libs/query.lib.php");
require_once("libs/post.lib.php");

// This variable is used to time how long it takes to generate the view
$viewStart = microt(true);

/* The ViewPoint accepts the following variables via HTTP GET.  Note that none
   of these variables are case-sensitive and that the order in which they are
   passed makes no difference.
   
   show     Specifies the name of the channel and the view which the user is 
            attempting to view.  The format of the parameter is
            "channelName@viewName".  "viewName" is optional and, if it is
            omitted, the default view for the channel specified will be used.
            view
            channel
            
   params   For dynamic views, specifies the search criteria or search terms
      
   limits   For dynamic views, specifies the limit and offset of the search
            results.  The format of this parameter is "offset.limit".
                    
  Note that all parameters are optional.  
*/

// If parameters were passed via GET...
if (count($_GET) > 0) {
  // ..get them
  $clientView = $_GET;
  // Change all of the key names to lowercase
  $clientView = array_change_key_case($clientView);
  // Determine whether or not a view and/or channel were specified
  if (isset($clientView["show"])) {
    // Expand the view name, if given
    list($clientView["channel"], $clientView["view"]) = 
      explode("@", $clientView["show"]);
  }
  DebugOut("ViewPoint", 
    "Working with the following parameters:<br />",
    implode_assoc($clientView));
}

// Connect to the site database
$thisSite = new SITE();

// Start (or continue) a session
$thisSession = new SITE_SESSION($thisSite);

// Set this in case we encounter an error
$_SESSION["LastAction"] = "ERROR";

// If no channel was specified, we go with the default channel
if (!isset($clientView["channel"]) || empty($clientView["channel"])) {
  // Retrieve the default channel name
  $clientView["channel"] = $thisSite->Setting("Defaults", "DefaultChannel");
  // If we failed to retrieve the default channel name, there's an error!
  if ($clientView["channel"] < 0) DiePrettily(seSiteConfig, array( 
    "ViewPoint",
    " - The site configuration fails to specify a default channel for ",
    "visitors to view."));
}

// Retrieve information about the specified channel
$thisChannel = new SITE_CHANNEL($thisSite);
$result = $thisChannel->Get($clientView["channel"]);
// If the channel doesn't exist, die with a pretty error message
if ($result < 0) DiePrettily(seResourceNotFound, array(
  "ViewPoint",
  " - The system was unable to find the channel specified: ", 
  $clientView["channel"]));
  
/* Retrieve information about the current user */
$thisUser = new SITE_USER($thisSite);
// Get the user's information
$result = $thisUser->Get($_SESSION["UserName"]);
// If the user was not found, indicate an error
if ($result < 0)
  DiePrettily(seResourceNotFound, array(
  "ViewPoint",
  " - The system was unable to find the user specified by the current ",
  "session: ", $_SESSION["UserName"]));
// Update the user's last visit timestamp
$now = $thisSite->DataSource();
$now = $now->TimeValue();
$thisUser->Set("LastVisit", $now);

/* Find out if the user can access this channel */
$subscription = $thisUser->Subscription($thisChannel);

/* If we don't have an array containing information about the group
   subscription to this channel, there's a problem (the user probably cannot
   access the channel)
*/
if (!is_array($subscription)) DiePrettily($subscription, array( 
  "ViewPoint",
  " - The system was unable to find a subscription for the current user -- ", 
  $thisUser->Setting("UserName"), " -- that would grant access to: ", 
  $clientView["channel"]));
    
// If the access rights have not been specified, indicate an error
if (empty($subscription["Rights"]))
  DiePrettily(seResourceNotFound, array(
  "ViewPoint",
  " - The system was unable to find a subscription for the current user: ",
  $thisUser->Setting("UserName")));

/* If the access rights do not include the authorization to read posts in the
   channel, then indicate an error
*/
if (strpos($subscription["Rights"], "r") === FALSE)
  DiePrettily(seResourceForbidden, array( 
  "ViewPoint",
  " - The system was unable to find a subscription for the current user -- ",
  $thisUser->Setting("UserName"), " -- that would grant access to: ",
  $clientView["channel"]));
  
/* Set the last subscription used by the user in the session variables so that
   others can use it
*/

$_SESSION["Subscription"] = implode_assoc($subscription);
$_SESSION["UserRights"] = $subscription["Rights"];

/* Retrieve the view to use for this user (whether it be the default view for
   the channel or a user-specific view)
*/
$thisView = new SITE_VIEW($thisSite, $thisSession, $thisChannel, $thisUser);

// If no view was specified, we use the default view for the channel
if (!isset($clientView["view"]) || empty($clientView["view"]))
  $clientView["view"] = $thisChannel->Setting("DefaultView");

// If a view was specified, see if it can be displayed
$result = $thisView->Get($clientView["view"]);
// If not, let the user know
if ($result < 0) DiePrettily($result, array(
    "ViewPoint",
    " - The system was unable to apply the requested view -- ",
    $clientView["view"], " -- to the channel: ", $clientView["channel"]));

// Set the last action
$_SESSION["LastAction"] = "VIEW";
// Set the last view generated for the user
$_SESSION["LastView"] = $thisView->Setting("ViewName");
// Set the last channel shown
$_SESSION["LastChannel"] = $thisChannel->Setting("ChannelName");

/* Generate and display the view */
$result = $thisView->Show();

if ($result < 0) DiePrettily(seResource, array(
  "ViewPoint",
  " - The system was unable to display the requested view -- ",
  $thisView->Setting("ViewName"),
  " -- for the channel: ",
  $thisChannel->Setting("ChannelName")));

/* STATISTICS TRACKING */
$viewCount = $thisSite->Setting("Statistics", "ViewCount") + 1;
// Increment the view counter
$thisSite->Set("Statistics", "ViewCount", $viewCount);

// Retrieve the total time spent rendering all views
$totalRenderTime = $thisSite->Setting("Statistics", "TotalRenderTime");

// Finish timing the view
$viewTime = round(microt(true) - $viewStart, 3);
// Update the total time spent rendering all views
$totalRenderTime += $viewTime;
$thisSite->Set("Statistics", "TotalRenderTime", $totalRenderTime);
// Stash the view generation time
$thisSite->Set("Statistics", "RenderTime", $viewTime);
// Stash the average render time per view
$avgRenderTime = round($totalRenderTime/$viewCount, 4);
$thisSite->Set("Statistics", "AvgRenderTime", $avgRenderTime);
// Stash the longest render time, if necessary
$longestRenderTime = $thisSite->Setting("Statistics", "LongestRenderTime");
if ($viewTime > $longestRenderTime) {
  $thisSite->Set("Statistics", "LongestRenderTime", $viewTime);
  $thisSite->Set("Statistics", "LongestRenderView", $thisView->Setting("ViewName"));
}
// Stash the shortest render time, if necessary
$shortestRenderTime = $thisSite->Setting("Statistics", "ShortestRenderTime");
if ($viewTime < $shortestRenderTime) {
  $thisSite->Set("Statistics", "ShortestRenderTime", $viewTime);
  $thisSite->Set("Statistics", "ShortestRenderView", $thisView->Setting("ViewName"));
}

/* Clean up */
$thisView->__destruct();
$thisUser->__destruct();
$thisChannel->__destruct();
$thisSession->__destruct();
// Flush all site information
$thisSite->__destruct();
/* If debugging is enabled, output the debugging information into a separate
   window
*/
ShowDebugConsole();

// Flush the output buffer
header("HTTP/1.1 200 OK");
ob_end_flush();

?>
